@@ -1,12 +1,12 @@
 from .schemdraw import Drawing, use, config, theme
-from .segments import Segment, SegmentCircle, SegmentArc, SegmentText, SegmentPoly, SegmentArrow
+from .segments import Segment, SegmentCircle, SegmentArc, SegmentText, SegmentPoly, SegmentBezier, SegmentArrow
 from .transform import Transform
 from .types import ImageFormat
 from .backends.svg import settextmode
 
 __all__ = [
     "Drawing", "use", "config", "theme", "Segment", "SegmentCircle", "SegmentArc", "SegmentText",
-    "SegmentPoly", "SegmentArrow", "Transform", "ImageFormat", "settextmode"
+    "SegmentPoly", "SegmentBezier", "Transform", "ImageFormat", "settextmode"
 ]
 
-__version__ = '0.12'
+__version__ = '0.13'
