@@ -2,12 +2,6 @@
 
 from __future__ import annotations
 
-import sys
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
-
 from typing import Sequence, Optional
 from xml.etree import ElementTree as ET
 
@@ -22,17 +16,19 @@ try:
 except ImportError:
     ziamath = None  # type: ignore
 
-from ..types import Capstyle, Joinstyle, Linestyle, BBox
+from ..types import Capstyle, Joinstyle, Linestyle, BBox, Halign, Valign, RotationMode, TextMode
 from ..util import Point
 from . import svgtext
 
-TextModeType = Literal['path', 'text']
 
-textmode: TextModeType = 'path' if ziamath is not None else 'text'
+textmode: TextMode = 'path' if ziamath is not None else 'text'
 svg2mode: bool = True
-    
 
-def settextmode(mode: TextModeType, svg2: bool=True) -> None:
+hatchpattern = '''<defs><pattern id="hatch" patternUnits="userSpaceOnUse" width="4" height="4">
+<path d="M-1,1 l2,-2 M0,4 l4,-4 M3,5 l2,-2" style="stroke:black; stroke-width:.5" /></pattern></defs>'''
+
+
+def settextmode(mode: TextMode, svg2: bool=True) -> None:
     ''' Set the mode for rendering text in the SVG backend.
 
         In 'text' mode, text is drawn as SVG <text> elements
@@ -70,14 +66,17 @@ inline = isnotebook()
 
 def getstyle(color: str=None, ls: Linestyle=None, lw: float=None,
              capstyle: Capstyle=None, joinstyle: Joinstyle=None,
-             fill: str=None) -> str:
+             fill: str=None, hatch: bool=False) -> str:
     ''' Get style for svg element. Leave empty if property matches default '''
     # Note: styles are added to every SVG element, rather than in a global <style>
     # tag, since multiple images in one HTML page may share <styles>.
     s = ''
     if color:
         s += f'stroke:{color};'
-    s += f'fill:{str(fill).lower()};'
+    if hatch:
+        s += 'fill:url(#hatch);'
+    else:
+        s += f'fill:{str(fill).lower()};'
     if lw:
         s += f'stroke-width:{lw};'
     dash = {'--': '7.4,3.2',
@@ -108,7 +107,7 @@ def text_size(text: str, font: str='Arial', size: float=16) -> tuple[float, floa
             text: string to calculate
             font: Font family
             size: Font size in points
-        
+
         Returns:
             width, height, linespacing
     '''
@@ -118,7 +117,7 @@ def text_size(text: str, font: str='Arial', size: float=16) -> tuple[float, floa
 
         if font.lower() in ['sans-serif', 'Arial']:
             font = 'sans'
-        
+
         lines = text.splitlines()
         maths = []
         for line in lines:
@@ -142,6 +141,8 @@ class Figure:
     '''
     def __init__(self, bbox: BBox, **kwargs):
         self.svgelements: list[tuple[int, ET.Element]] = []  # (zorder, element)
+        self.hatch: bool = False
+        self.clips: dict[BBox, str] = {}
         self.showframe = kwargs.get('showframe', False)
         self.scale = 64.8 * kwargs.get('inches_per_unit', .5)  # Magic scale factor that matches what MPL did
         self.set_bbox(bbox)
@@ -155,18 +156,34 @@ class Figure:
         self.pxwidth = max(5, self.pxwidth)
         self.pxheight = max(5, self.pxheight)
 
-    def xform(self, x: float, y: float) -> tuple[float, float]:
+    def xform(self, x: float, y: float) -> Point:
         ''' Convert x, y in user coords to svg pixel coords '''
-        return x*self.scale, -y*self.scale
+        return Point((x*self.scale, -y*self.scale))
 
     def bgcolor(self, color: str) -> None:
         ''' Set background color of drawing '''
         self._bgcolor = color
 
+    def addclip(self, et: ET.Element, bbox: Optional[BBox]):
+        ''' Add clip path to the element '''
+        if bbox is not None:
+            if bbox in self.clips:
+                clipid = self.clips[bbox]
+            else:
+                clipid = str(len(self.clips))
+                self.clips[bbox] = clipid
+
+                x0, y0 = self.xform(bbox.xmin, bbox.ymin)
+                x1, y1 = self.xform(bbox.xmax, bbox.ymax)
+                clip = ET.fromstring(f'''<defs>
+    <clipPath id="clip{clipid}"><rect x="{x0-1}" y="{y0-1}" width="{x1-x0+2}" height="{y1-y0+2}" /></clipPath></defs>''')
+                self.svgelements.append((0, clip))
+            et.set('clip-path', f'url(#clip{clipid})')
+
     def plot(self, x: Sequence[float], y: Sequence[float],
              color: str='black', ls: Linestyle='-', lw: float=2,
              fill: str='none', capstyle: Capstyle='round',
-             joinstyle: Joinstyle='round', zorder: int=2) -> None:
+             joinstyle: Joinstyle='round', clip: BBox=None, zorder: int=2) -> None:
         ''' Plot a path '''
         et = ET.Element('path')
         d = 'M {},{} '.format(*self.xform(x[0], y[0]))
@@ -183,17 +200,19 @@ class Figure:
         et.set('d', d)
         et.set('style', getstyle(color=color, ls=ls, lw=lw, capstyle=capstyle,
                                  joinstyle=joinstyle, fill=fill))
+        self.addclip(et, clip)
         self.svgelements.append((zorder, et))
 
     def text(self, s: str, x: float, y: float, color: str='black',
              fontsize: float=14, fontfamily: str='sans-serif',
-             rotation: float=0, halign: Literal['left', 'center', 'right']='center',
-             valign: Literal['top', 'center', 'bottom']='center',
-             rotation_mode: Literal['anchor', 'default']='anchor', zorder: int=3) -> None:
+             rotation: float=0, halign: Halign='center',
+             valign: Valign='center',
+             rotation_mode: RotationMode='anchor', clip: BBox=None, zorder: int=3) -> None:
         ''' Add text to the figure '''
         if s == '':
             return
 
+        y += fontsize/72*2/6  # A bit of baseline shift to match MPL
         x, y = self.xform(x, y)
 
         if ziamath and textmode == 'path':
@@ -235,32 +254,29 @@ class Figure:
                                          halign=halign, valign=valign, color=color,
                                          rotation=rotation, rotation_mode=rotation_mode,
                                          testmode=False)
-
+        self.addclip(texttag, clip)
         self.svgelements.append((zorder, texttag))
 
     def poly(self, verts: Sequence[Sequence[float]], closed: bool=True,
              color: str='black', fill: str='none', lw: float=2,
-             ls: Linestyle='-', capstyle: Capstyle='round',
-             joinstyle: Joinstyle='round', zorder: int=1) -> None:
+             ls: Linestyle='-', hatch: bool=False, capstyle: Capstyle='round',
+             joinstyle: Joinstyle='round', clip: BBox=None, zorder: int=1) -> None:
         ''' Draw a polygon '''
-        if not closed:
-            x = [v[0] for v in verts]
-            y = [v[1] for v in verts]
-            self.plot(x, y, color=color, fill=fill, lw=lw, ls=ls,
-                      capstyle=capstyle, joinstyle=joinstyle, zorder=zorder)
-        else:
-            et = ET.Element('polygon')
-            points = ''
-            for xx, yy in verts:
-                xx, yy = self.xform(xx, yy)
-                points += '{},{} '.format(xx, yy)
-            et.set('points', points)
-            et.set('style', getstyle(color=color, ls=ls, lw=lw, capstyle=capstyle,
-                                     joinstyle=joinstyle, fill=fill))
-            self.svgelements.append((zorder, et))
+        et = ET.Element('polyline') if not closed else ET.Element('polygon')
+        points = ''
+        for xx, yy in verts:
+            xx, yy = self.xform(xx, yy)
+            points += '{},{} '.format(xx, yy)
+        et.set('points', points)
+        et.set('style', getstyle(color=color, ls=ls, lw=lw, capstyle=capstyle,
+                                 joinstyle=joinstyle, fill=fill, hatch=hatch))
+        self.addclip(et, clip)
+        self.svgelements.append((zorder, et))
+        if hatch:
+            self.hatch = True
 
     def circle(self, center: Sequence[float], radius: float, color: str='black',
-               fill: str='none', lw: float=2, ls: Linestyle='-', zorder: int=1) -> None:
+               fill: str='none', lw: float=2, ls: Linestyle='-', clip: BBox=None, zorder: int=1) -> None:
         ''' Draw a circle '''
         x, y = self.xform(*center)
         radius = radius * self.scale
@@ -269,31 +285,32 @@ class Figure:
         et.set('cy', str(y))
         et.set('r', str(radius))
         et.set('style', getstyle(color=color, lw=lw, ls=ls, fill=fill))
+        self.addclip(et, clip)
         self.svgelements.append((zorder, et))
 
-    def arrow(self, x: float, y: float, dx: float, dy: float,
-              headwidth: float=.2, headlength: float=.2,
-              color: str='black', lw: float=2, zorder: int=1) -> None:
-        ''' Draw an arrow '''
-        x, y = self.xform(x, y)
-        dx, dy = dx*self.scale, dy*self.scale
-        headwidth = headwidth*self.scale
-        headlength = headlength*self.scale
+    def arrow(self, xy: Sequence[float], theta: float,
+              arrowwidth: float=.2, arrowlength: float=.2,
+              color: str='black', lw: float=2, clip: BBox=None, zorder: int=1) -> None:
+        ''' Draw an arrowhead '''
+        x, y = self.xform(*xy)
+        dx = arrowlength/2 * math.cos(math.radians(theta)) * self.scale
+        dy = arrowlength/2 * math.sin(math.radians(theta)) * self.scale
+        arrowwidth = arrowwidth*self.scale
+        arrowlength = arrowlength*self.scale
 
         # Draw arrow as path
-        head = Point((x+dx, y-dy))
-        tail = Point((x, y))
+        head = Point((x, y))
+        tail = Point((x-dx, y+dy))
         fullen = math.sqrt(dx**2 + dy**2)
         theta = -math.degrees(math.atan2(dy, dx))
 
-        finc = Point((fullen - headlength, 0)).rotate(theta) + tail
-        fin1 = Point((fullen - headlength, headwidth/2)).rotate(theta) + tail
-        fin2 = Point((fullen - headlength, -headwidth/2)).rotate(theta) + tail
+        fin1 = Point((fullen - arrowlength, arrowwidth/2)).rotate(theta) + tail
+        fin2 = Point((fullen - arrowlength, -arrowwidth/2)).rotate(theta) + tail
 
         # Shrink arrow head by lw so it points right at the line
-        head = Point((head[0] - lw * math.cos(math.radians(theta)),
-                      head[1] - lw * math.sin(math.radians(theta))))
-        
+        head = Point((head[0] - lw *2* math.cos(math.radians(theta)),
+                      head[1] - lw *2* math.sin(math.radians(theta))))
+
         et1 = ET.Element('path')
         d = f'M {head[0]} {head[1]} '
         d += f'L {fin1[0]} {fin1[1]} '
@@ -301,20 +318,55 @@ class Figure:
         et1.set('d', d)
         et1.set('style', getstyle(color=color, lw=0, capstyle='butt',
                                   joinstyle='miter', fill=color))
-
-        et2 = ET.Element('path')
-        d = f'M {finc[0]} {finc[1]} '
-        d += f'L {tail[0]} {tail[1]} Z'
-        et2.set('d', d)
-        et2.set('style', getstyle(color=color, lw=lw, capstyle='butt',
-                                  joinstyle='miter', fill=color))
+        self.addclip(et1, clip)
         self.svgelements.append((zorder, et1))
-        self.svgelements.append((zorder, et2))
+
+    def bezier(self, p: Sequence[Point], color: str='black',
+               lw: float=2, ls: Linestyle='-', capstyle: Capstyle='round', zorder: int=1,
+               arrow: str=None, arrowlength=.25, arrowwidth=.15, clip: BBox=None) -> None:
+        ''' Draw a cubic or quadratic bezier '''
+        # Keep original points for arrow head
+        # and adjust points for line so they don't extrude from arrows.
+        lpoints = list(p)
+        if arrow is not None:
+            if '<' in arrow:
+                th1 = math.atan2(p[0].y - p[1].y, p[0].x - p[1].x)
+                lpoints[0] = Point((p[0].x - math.cos(th1) * arrowlength/2,
+                                    p[0].y - math.sin(th1) * arrowlength/2))
+            if '>' in arrow:
+                th2 = math.atan2(p[-1].y - p[-2].y, p[-1].x - p[-2].x)
+                lpoints[-1] = Point((p[-1].x - math.cos(th2) * arrowlength/2,
+                                    p[-1].y - math.sin(th2) * arrowlength/2))
+
+        lpoints = [self.xform(*p0) for p0 in lpoints]
+        order = 'C' if len(p) == 4 else 'Q'
+
+        et = ET.Element('path')
+        path = f'M {lpoints[0][0]} {lpoints[0][1]} {order}'
+        for p0 in lpoints[1:]:
+            path += f' {p0[0]} {p0[1]}'
+        et.set('d', path)
+        et.set('style', getstyle(color=color, ls=ls, lw=lw, capstyle=capstyle))
+        self.addclip(et, clip)
+        self.svgelements.append((zorder, et))
+
+        if arrow is not None:
+            # Note: using untransformed bezier control points here
+            if '<' in arrow:
+                delta = p[0] - p[1]
+                theta = math.degrees(math.atan2(delta.y, delta.x))
+                self.arrow(p[0], theta, color=color, lw=1, zorder=zorder,
+                           clip=clip, arrowlength=arrowlength, arrowwidth=arrowwidth)
+            if '>' in arrow:
+                delta = p[-1] - p[-2]
+                theta = math.degrees(math.atan2(delta.y, delta.x))
+                self.arrow(p[-1], theta, color=color, lw=1, zorder=zorder,
+                           clip=clip, arrowlength=arrowlength, arrowwidth=arrowwidth)
 
     def arc(self, center: Sequence[float], width: float, height: float,
             theta1: float=0, theta2: float=90, angle: float=0,
             color: str='black', lw: float=2, ls: Linestyle='-', zorder: int=1,
-            arrow: bool=None) -> None:
+            arrow: str=None, clip: BBox=None) -> None:
         ''' Draw an arc or ellipse, with optional arrowhead '''
         centerx, centery = self.xform(*center)
         width, height = width*self.scale, height*self.scale
@@ -330,13 +382,13 @@ class Figure:
         while t1 < t2:
             t1 += 2*math.pi
 
-        startx = (centerx + width/2 * math.cos(t2)*math.cos(anglerad) 
+        startx = (centerx + width/2 * math.cos(t2)*math.cos(anglerad)
                   - height/2 * math.sin(t2)*math.sin(anglerad))
-        starty = (centery + width/2 * math.cos(t2)*math.sin(anglerad) 
+        starty = (centery + width/2 * math.cos(t2)*math.sin(anglerad)
                   + height/2 * math.sin(t2)*math.cos(anglerad))
         endx = (centerx + width/2 * math.cos(t1)*math.cos(anglerad) 
                 - height/2 * math.sin(t1)*math.sin(anglerad))
-        endy = (centery + width/2 * math.cos(t1)*math.sin(anglerad) 
+        endy = (centery + width/2 * math.cos(t1)*math.sin(anglerad)
                 + height/2 * math.sin(t1)*math.cos(anglerad))
 
         startx, starty = round(startx, 2), round(starty, 2)
@@ -356,6 +408,7 @@ class Figure:
             et.set('stroke', color)
             et.set('stroke-width', str(lw))
             et.set('fill', 'none')
+            self.addclip(et, clip)
             self.svgelements.append((zorder, et))
 
         else:
@@ -367,38 +420,47 @@ class Figure:
             et.set('stroke', color)
             et.set('stroke-width', str(lw))
             et.set('fill', 'none')
+            self.addclip(et, clip)
             self.svgelements.append((zorder, et))
 
         if arrow is not None:
+            # Note: This arrowhead's TAIL is located at the endpoint of the
+            # arc curve. The arrow points beyond the arc's theta.
             # Back to user coordinates
             width, height = width/self.scale, height/self.scale
             angle = -angle
             theta1 = -math.degrees(theta1)
             theta2 = -math.degrees(theta2)
 
-            headlength = .25
-            headwidth = .15
+            arrowlength = .25
+            arrowwidth = .15
 
             x, y = math.cos(math.radians(theta2)), math.sin(math.radians(theta2))
             th2 = math.degrees(math.atan2((width/height)*y, x))
             x, y = math.cos(math.radians(theta1)), math.sin(math.radians(theta1))
             th1 = math.degrees(math.atan2((width/height)*y, x))
-            if arrow == 'ccw':
-                dx = math.cos(math.radians(th2+90)) * headlength
-                dy = math.sin(math.radians(theta2+90)) * headlength
+            if arrow in ['ccw', 'end', 'both'] or '>' in arrow:
+                dx = math.cos(math.radians(th2+90)) * arrowlength
+                dy = math.sin(math.radians(th2+90)) * arrowlength
                 xy = Point((center[0] + width/2*math.cos(math.radians(th2)),
                             center[1] + height/2*math.sin(math.radians(th2))))
-            else:
-                dx = -math.cos(math.radians(th1+90)) * headlength
-                dy = -math.sin(math.radians(th1+90)) * headlength
+                xy = Point(xy).rotate(angle, center)
+                darrow = Point((dx, dy)).rotate(angle)
+                theta = math.degrees(math.atan2(darrow.y, darrow.x))
+                self.arrow(xy+darrow, theta, arrowwidth=arrowwidth,
+                           arrowlength=arrowlength, color=color, lw=1, zorder=zorder)
+
+            if arrow in ['cw', 'start', 'both'] or '<' in arrow:
+                dx = -math.cos(math.radians(th1+90)) * arrowlength
+                dy = -math.sin(math.radians(th1+90)) * arrowlength
                 xy = Point((center[0] + width/2*math.cos(math.radians(th1)),
                             center[1] + height/2*math.sin(math.radians(th1))))
 
-            xy = Point(xy).rotate(angle, center)
-            darrow = Point((dx, dy)).rotate(angle)
-
-            self.arrow(xy[0], xy[1], darrow[0], darrow[1], headwidth=headwidth,
-                       headlength=headlength, color=color, lw=1, zorder=zorder)
+                xy = Point(xy).rotate(angle, center)
+                darrow = Point((dx, dy)).rotate(angle)
+                theta = math.degrees(math.atan2(darrow.y, darrow.x))
+                self.arrow(xy+darrow, theta, arrowwidth=arrowwidth,
+                           arrowlength=arrowlength, color=color, lw=1, zorder=zorder)
 
     def save(self, fname: str, **kwargs) -> None:
         ''' Save the figure to a file '''
@@ -420,6 +482,9 @@ class Figure:
         if self._bgcolor:
             svg.set('style', f'background-color:{self._bgcolor};')
 
+        if self.hatch:
+            svg.append(ET.fromstring(hatchpattern))
+
         if self.showframe:
             rect = ET.SubElement(svg, 'rect')
             rect.set('x', str(x0))
