@@ -7,7 +7,7 @@ from dataclasses import dataclass
 import warnings
 import math
 
-from ..segments import Segment, SegmentPoly, SegmentText, SegmentCircle, SegmentArc, SegmentArrow, BBox, SegmentType
+from ..segments import SegmentText, BBox, SegmentType
 from ..transform import Transform
 from .. import util
 from ..util import Point
@@ -39,7 +39,7 @@ class Element:
 
         Keyword Arguments are equivalent to calling
         setter methods.
-        
+
         Args:
             d: Drawing direction ('up', 'down', 'left', 'right')
 
@@ -62,7 +62,7 @@ class Element:
         self._localshift: XY = Point((0, 0))
         self._userlabels: list[Label] = []
 
-        self.anchors: MutableMapping[str, Union[Point, tuple[float,float]]] = {}     # Untransformed anchors
+        self.anchors: MutableMapping[str, Union[Point, tuple[float, float]]] = {}  # Untransformed anchors
         self.absanchors: MutableMapping[str, Any] = {}  # Transformed, absolute anchors
         self.segments: list[SegmentType] = []
         self.transform = Transform(0, [0, 0])
@@ -97,14 +97,14 @@ class Element:
     def left(self) -> 'Element':
         ''' Set the direction to left '''
         if 'd' in self._userparams:
-            warnings.warn(f"Duplicated direction parameter in element. `{self._userparams['d']}` changed to `left`.")        
+            warnings.warn(f"Duplicated direction parameter in element. `{self._userparams['d']}` changed to `left`.")
         self._userparams['d'] = 'left'
         return self
 
     def right(self) -> 'Element':
         ''' Set the direction to right '''
         if 'd' in self._userparams:
-            warnings.warn(f"Duplicated direction parameter in element. `{self._userparams['d']}` changed to `right`.")        
+            warnings.warn(f"Duplicated direction parameter in element. `{self._userparams['d']}` changed to `right`.")
         self._userparams['d'] = 'right'
         return self
 
@@ -115,6 +115,13 @@ class Element:
         self._userparams['theta'] = theta
         return self
 
+    def drop(self, drop: Union[str, Point]) -> 'Element':
+        ''' Set the drop position - where to leave the current drawing position after
+            placing this element
+        '''
+        self._userparams['drop'] = drop
+        return self
+
     def at(self, xy: XY | tuple['Element', str]) -> 'Element':
         ''' Set the element xy position
 
@@ -357,13 +364,20 @@ class Element:
         self.absanchors['xy'] = self.transform.transform((0, 0))
 
         drop = self._cparams.get('drop', None)
-        if drop is None or not self._cparams.get('move_cur', True):
+        if drop in self.anchors:
+            # User specified as anchor position
+            self.absdrop = self.transform.transform(self.anchors[drop]), theta
+        elif drop is not None and self._cparams.get('move_cur', True):
+            if self.params.get('droptheta', None):
+                # Element-specified drop angle
+                self.absdrop = self.transform.transform(drop), self.params.get('droptheta')
+            elif self.params.get('theta', None) == 0:
+                # Element-specified theta of 0, don't change theta
+                self.absdrop = self.transform.transform(drop), dwgtheta
+            else:
+                self.absdrop = self.transform.transform(drop), theta
+        else:  # drop is None:
             self.absdrop = Point(dwgxy), dwgtheta
-        elif self.params.get('theta', None) == 0:
-            # Element def specified theta = 0, don't change
-            self.absdrop = self.transform.transform(drop), dwgtheta
-        else:
-            self.absdrop = self.transform.transform(drop), theta
         return self.absdrop
 
     def get_bbox(self, transform=False, includetext=True):
@@ -466,7 +480,9 @@ class Element:
             elif loc == 'right':
                 loc = 'left'
 
-        if align is None and loc == 'center' and isinstance(label, (list, tuple)):
+        if align is None and 'lblalign' in self._cparams:
+            align = self._cparams['lblalign']
+        elif align is None and loc == 'center' and isinstance(label, (list, tuple)):
             align = ('center', 'center')
         elif align is None:
             align = (None, None)
@@ -495,7 +511,6 @@ class Element:
 
             if loc and loc in self.anchors:
                 x1, y1, x2, y2 = self.get_bbox(includetext=False)
-
                 if (math.isclose(self.anchors[loc][0], x1, abs_tol=.15) or
                    math.isclose(self.anchors[loc][0], x2, abs_tol=.15) or
                    math.isclose(self.anchors[loc][1], y1, abs_tol=.15) or
@@ -594,7 +609,11 @@ class Element:
 
         elif isinstance(label, str):
             # Place in center
-            if loc == 'top':
+            if loc in self.anchors:
+                xy = Point(self.anchors[loc])  # type: ignore
+                ofst = Point((0, ofst)) if not isinstance(ofst, (list, tuple)) else Point(ofst)
+                xy = Point(xy)
+            elif loc == 'top':
                 ofst = Point((0, ofst)) if not isinstance(ofst, (list, tuple)) else Point(ofst)
                 xy = Point(((xmax+xmin)/2, ymax))
             elif loc == 'bottom':
@@ -609,10 +628,6 @@ class Element:
             elif loc == 'center':
                 ofst = Point((0, ofst)) if not isinstance(ofst, (list, tuple)) else Point(ofst)
                 xy = Point(((xmax+xmin)/2, (ymax+ymin)/2))
-            elif loc in self.anchors:
-                xy = Point(self.anchors[loc])  # type: ignore
-                ofst = Point((0, ofst)) if not isinstance(ofst, (list, tuple)) else Point(ofst)
-                xy = Point(xy)
             else:
                 raise ValueError('Undefined location {}'.format(loc))
             xy = xy + ofst
