@@ -4,10 +4,11 @@ from __future__ import annotations
 from typing import Sequence
 import math
 
-from ..segments import Segment, SegmentArrow, SegmentCircle, SegmentArc, SegmentPoly
+from ..segments import Segment, SegmentCircle, SegmentArc, SegmentPoly, SegmentBezier
 from .elements import Element, Element2Term
 from .twoterm import gap
 from ..types import XY, Point, Arcdirection, BilateralDirection
+from .. import util
 
 
 class Line(Element2Term):
@@ -33,43 +34,18 @@ class Arrow(Line):
         self.double = double
         self.headlength = headlength
         self.headwidth = headwidth
-        self.segments.append(SegmentArrow((-headlength, 0), (0, 0),
-                                          ref='end',
-                                          headwidth=headwidth,
-                                          headlength=headlength))
-        if self.double:
-            self.segments.append(SegmentArrow((headlength, 0), (0, 0),
-                                              ref='start',
-                                              headwidth=headwidth,
-                                              headlength=headlength))
+
         # Explicitly define center so reverses work
         self.anchors['center'] = (0, 0)
-        
-    def _place(self, xy, theta, **dwgparams):
-        ''' Shorten the underlying line so it doesn't stick out
-            from arrowhead
-        '''
-        result = super()._place(xy, theta, **dwgparams)
-        line = self.segments[0]  # The base line which needs to be shortened
-        xy1 = line.path[0]
-        xy2 = line.path[-1]
-        dx = xy2[0] - xy1[0]
-        dy = xy2[1] - xy1[1]
-        theta = math.atan2(dy, dx)
-        newhead = xy2
-        newtail = xy1
-        reverse = self._cparams.get('reverse')
 
-        if reverse or self.double:
-            newtail = (xy1[0] + self.headlength * math.cos(theta),
-                       xy1[1] + self.headlength * math.sin(theta))
-        
-        if (not reverse) or self.double:
-            newhead = (xy2[0] - self.headlength * math.cos(theta),
-                       xy2[1] - self.headlength * math.sin(theta))
-            
-        line.path[0] = newtail
-        line.path[-1] = newhead
+    def _place(self, dwgxy, dwgtheta, **dwgparams):
+        ''' Add arrowhead to segment after it was extended '''
+        result = super()._place(dwgxy, dwgtheta, **dwgparams)
+        line = self.segments[0]  # The base line gets arrowheads
+        reverse = self._cparams.get('reverse')
+        arrow = '->' if not reverse else '<-'
+        arrow = '<->' if self.double else arrow
+        line.arrow = arrow
         return result
 
 
@@ -136,8 +112,8 @@ class Arrowhead(Element):
     ''' Arrowhead'''
     def __init__(self, *d, headwidth: float=.15, headlength: float=.2, **kwargs):
         super().__init__(*d, **kwargs)
-        self.segments.append(SegmentArrow(
-            (-.3, 0), (0, 0), headwidth=headwidth, headlength=headlength))
+        self.segments.append(Segment([
+            (-headlength, 0), (0, 0)], arrowwidth=headwidth, arrowlength=headlength, arrow='->'))
         self.anchors['start'] = (0, 0)
         self.anchors['center'] = (0, 0)
         self.anchors['end'] = (0, 0)
@@ -164,6 +140,281 @@ class DotDotDot(Element):
         self.params['drop'] = (2, 0)
 
 
+class Arc2(Element):
+    ''' Arc Element
+
+        Use `at` and `to` methods to define endpoints.
+
+        Arc2 is a quadratic Bezier curve with control point
+        halfway between the endpoints, generating a
+        symmetric 'C' curve.
+
+        Args:
+            k: Control point factor. Higher k means more curvature.
+
+        Anchors:
+            start
+            end
+            ctrl
+            mid
+    '''
+    def __init__(self, k=0.5, arrow=None, **kwargs):
+        super().__init__(**kwargs)
+        self.k = k
+        self.arrow = arrow
+
+    def to(self, xy: XY) -> 'Element':
+        ''' Specify ending position '''
+        self._userparams['to'] = xy
+        return self
+
+    def _place(self, dwgxy: XY, dwgtheta: float, **dwgparams) -> tuple[Point, float]:
+        ''' Calculate absolute placement of Element '''
+        self._dwgparams = dwgparams
+        if not self._cparams:
+            self._buildparams()
+
+        self.params['lblloc'] = 'center'
+        self.params['theta'] = 0
+        xy: Point = Point(self._cparams.get('at', dwgxy))
+        to: Point = Point(self._cparams.get('to', Point((xy.x+3, xy.y))))
+        dx = to.x - xy.x
+        dy = to.y - xy.y
+        pa = Point((dx/2-dy*self.k, dy/2+dx*self.k))
+        mid = Point((dx/2-dy*self.k/2, dy/2+dx*self.k/2))
+        self.segments.append(SegmentBezier(
+            (Point((0, 0)), pa, Point((dx, dy))), arrow=self.arrow),)
+
+        self.anchors['ctrl'] = pa
+        self.anchors['mid'] = mid
+        self.params['lblloc'] = 'mid'
+        self.anchors['start'] = Point((0, 0))
+        self.anchors['end'] = Point((dx, dy))
+        self.params['drop'] = Point((dx, dy))
+        valign = 'bottom'
+        halign = 'left'
+        hofst = 0.1
+        vofst = 0.1
+        if math.isclose(mid.y, pa.y, abs_tol=.01):
+            valign = 'center'
+            vofst = 0
+        elif mid.y > pa.y:
+            valign = 'top'
+            vofst = -0.1
+        if math.isclose(mid.x, pa.x, abs_tol=.01):
+            halign = 'center'
+            hofst = 0
+        elif mid.x > pa.x:
+            halign = 'right'
+            hofst = -0.1
+        self.params['lblofst'] = (hofst, vofst)
+        self.params['lblalign'] = (halign, valign)
+        return super()._place(dwgxy, dwgtheta, **dwgparams)
+
+
+class Arc3(Element):
+    ''' Arc Element
+
+        Use `at` and `to` methods to define endpoints.
+
+        Arc3 is a cubic Bezier curve. Control points are set
+        to extend the curve at the given angle for each endpoint.
+
+        Args:
+            k: Control point factor. Higher k means tighter curve.
+            th1: Angle at which the arc leaves start point
+            th2: Angle at which the arc leaves end point
+            arrow: arrowhead specifier, such as '->', '<-', or '<->'
+            arrowlength: Length of arrowhead
+            arrowwidth: Width of arrowhead
+
+        Anchors:
+            start
+            end
+            center
+            ctrl1
+            ctrl2
+    '''
+    def __init__(self, k=0.75, th1=0, th2=180, arrow=None,
+                 arrowlength=.25, arrowwidth=.2, **kwargs):
+        super().__init__(**kwargs)
+        self._userparams.setdefault('to', Point((1, 0)))
+        self.k = k
+        self.th1 = math.radians(th1)
+        self.th2 = math.radians(th2)
+        self.arrow = arrow
+        self.arrowlength = arrowlength
+        self.arrowwidth = arrowwidth
+
+    def to(self, xy: XY) -> 'Element':
+        ''' Specify ending position '''
+        self._userparams['to'] = xy
+        return self
+
+    def _place(self, dwgxy: XY, dwgtheta: float, **dwgparams) -> tuple[Point, float]:
+        ''' Calculate absolute placement of Element '''
+        self._dwgparams = dwgparams
+        if not self._cparams:
+            self._buildparams()
+
+        self.params['theta'] = 0
+        xy: Point = Point(self._cparams.get('at', dwgxy))
+        to: Point = Point(self._cparams.get('to', dwgxy))
+        dx = to.x - xy.x
+        dy = to.y - xy.y
+        pa1 = Point((dx*self.k*math.cos(self.th1),
+                     dy*self.k*math.sin(self.th1)))
+        pa2 = Point((dx+dx*self.k*math.cos(self.th2),
+                     dy+dy*self.k*math.sin(self.th2)))
+        self.segments.append(SegmentBezier((Point((0, 0)), pa1, pa2, Point((dx, dy))),
+                                           arrow=self.arrow,
+                                           arrowlength=self.arrowlength,
+                                           arrowwidth=self.arrowwidth))
+
+        # This sets reasonable label positions for ArcZ and ArcS modes.
+        # Other Arc3 maybe not.
+        self.anchors['center'] = Point((dx/2, dy/2))
+        self.anchors['start'] = Point((0, 0))
+        self.anchors['end'] = Point((dx, dy))
+        self.anchors['ctrl1'] = pa1
+        self.anchors['ctrl2'] = pa2
+        self.params['lblloc'] = 'center'
+        self.params['drop'] = Point((dx, dy))
+        halign = 'left'
+        valign = 'bottom'
+        if dy > 0:
+            valign = 'top'
+            self.params['lblofst'] = -0.1
+        if dx <= 0:
+            halign = 'right'
+        self.params['lblalign'] = (halign, valign)
+        return super()._place(dwgxy, dwgtheta, **dwgparams)
+
+
+class ArcZ(Arc3):
+    ''' Z-Curve Arc
+
+        Use `at` and `to` methods to define endpoints.
+
+        ArcZ approaches the endpoints horizontally, leading to
+        a 'Z' shaped curve
+
+        Args:
+            k: Control point factor. Higher k means tighter curve.
+            arrow: arrowhead specifier, such as '->', '<-', or '<->'
+            arrowlength: Length of arrowhead
+            arrowwidth: Width of arrowhead
+    '''
+    def __init__(self, k=0.75, arrow=None, arrowlength=.25, arrowwidth=.2, **kwargs):
+        super().__init__(k=k, arrow=arrow, th1=0, th2=180, arrowlength=arrowlength, arrowwidth=arrowwidth, **kwargs)
+
+
+class ArcN(Arc3):
+    ''' N-Curve Arc
+
+        Use `at` and `to` methods to define endpoints.
+
+        ArcN approaches the endpoints vertically, leading to
+        a 'N' shaped curve
+
+        Args:
+            k: Control point factor. Higher k means tighter curve.
+            arrow: arrowhead specifier, such as '->', '<-', or '<->'
+            arrowlength: Length of arrowhead
+            arrowwidth: Width of arrowhead
+    '''
+    def __init__(self, k=0.75, arrow=None, arrowlength=.25, arrowwidth=.2, **kwargs):
+        super().__init__(k=k, arrow=arrow, th1=90, th2=-90, arrowlength=arrowlength, arrowwidth=arrowwidth, **kwargs)
+
+
+class ArcLoop(Element):
+    ''' Loop Arc
+
+        Use `at` and `to` methods to define endpoints.
+
+        ArcLoop is an arc drawn as part of a circle.
+
+        Args:
+            radius: Radius of the arc
+            arrow: arrowhead specifier, such as '->', '<-', or '<->'
+            arrowlength: Length of arrowhead
+            arrowwidth: Width of arrowhead
+
+        Anchors:
+            start
+            end
+            mid
+            BL
+            BR
+            TL
+            TR
+    '''
+    def __init__(self, radius: float=0.6, arrow: str=None, arrowlength=.25, arrowwidth=.2, **kwargs):
+        super().__init__(**kwargs)
+        self.radius = radius
+        self.arrow = arrow
+        self.arrowlength = arrowlength
+        self.arrowwidth = arrowwidth
+
+    def to(self, xy: XY) -> 'Element':
+        ''' Specify ending position '''
+        self._userparams['to'] = xy
+        return self
+
+    def _place(self, dwgxy: XY, dwgtheta: float, **dwgparams) -> tuple[Point, float]:
+        ''' Calculate placement of Element '''
+        self._dwgparams = dwgparams
+        if not self._cparams:
+            self._buildparams()
+
+        self.params['theta'] = 0
+        xy: Point = Point(self._cparams.get('at', dwgxy))
+        to: Point = Point(self._cparams.get('to', dwgxy))
+
+        dx = to.x - xy.x
+        dy = to.y - xy.y
+
+        xa = dx/2
+        ya = dy/2
+        q = math.sqrt(dx**2 + dy**2)
+        try:
+            center = Point(((xa + math.sqrt(self.radius**2 - (q/2)**2) * (-dy/q)),
+                            (ya + math.sqrt(self.radius**2 - (q/2)**2) * (dx/q))))
+        except ValueError as err:
+            raise ValueError(f'No solution to ArcLoop with radius {self.radius}') from err
+
+        da = math.sqrt(xa**2 + ya**2)
+        thetaa = math.atan2(ya, xa)
+        thda = math.acos(da/self.radius)
+        theta1 = thetaa - thda
+        theta2 = thetaa + thda
+        t = util.linspace(math.pi+theta2, theta1)
+        x = [center.x + self.radius * math.cos(ti) for ti in t]
+        y = [center.y + self.radius * math.sin(ti) for ti in t]
+        self.segments.append(Segment(list(zip(x, y)), arrow=self.arrow,
+                                     arrowlength=self.arrowlength,
+                                     arrowwidth=self.arrowwidth))
+        mid = len(x)//2
+        self.anchors['start'] = Point((0, 0))
+        self.anchors['end'] = Point((x[-1], y[-1]))
+        self.anchors['mid'] = Point((x[mid], y[mid]))
+        self.anchors['BL'] = Point((min(x), min(y)))
+        self.anchors['BR'] = Point((max(x), min(y)))
+        self.anchors['TL'] = Point((min(x), max(y)))
+        self.anchors['TR'] = Point((max(x), max(y)))
+        self.params['lblloc'] = 'mid'
+        self.params['drop'] = to
+        valign = 'bottom'
+        halign = 'left'
+        if y[mid] < y[0]:
+            valign = 'top'
+            self.params['lblofst'] = -.1
+        if x[mid] < x[0] and x[mid] < x[-1]:
+            halign = 'right'
+        self.params['lblalign'] = (halign, valign)
+        return super()._place(dwgxy, dwgtheta, **dwgparams)
+
+
 class Label(Element):
     ''' Label element.
 
@@ -232,12 +483,11 @@ class CurrentLabel(Element):
         if reverse:
             a, b = b, a
 
-        self.segments.append(SegmentArrow(
-            a, b, headwidth=.2, headlength=.3))
+        self.segments.append(Segment((a, b), arrow='->', arrowwidth=.2, arrowlength=.3))
 
     def at(self, xy: XY | Element) -> 'Element':  # type: ignore[override]
         ''' Specify CurrentLabel position.
-        
+
             If xy is an Element, arrow will be centered
             along element and its color will also be
             inherited.
@@ -287,8 +537,8 @@ class CurrentLabelInline(Element):
             x = -x
             dx = -dx
 
-        self.segments.append(SegmentArrow(
-            (x, 0), (x+dx, 0), headwidth=headwidth, headlength=headlength))
+        self.segments.append(Segment(((x, 0), (x+dx, 0)), arrow='->',
+                                     arrowwidth=headwidth, arrowlength=headlength))
 
     def at(self, xy: XY | Element) -> 'Element':  # type: ignore[override]
         ''' Specify CurrentLabelInline position.
@@ -305,7 +555,7 @@ class CurrentLabelInline(Element):
             super().at(xy.center)
             self.theta(xy.transform.theta)
             if 'color' in xy._userparams:
-                self.color(xy._userparams.get('color'))            
+                self.color(xy._userparams.get('color'))
         else:
             super().at(xy)
         return self
