@@ -6,7 +6,7 @@ from typing import Sequence
 
 from .elements import Element, Element2Term, gap
 from ..util import Point, linspace
-from ..segments import Segment, SegmentArrow, SegmentArc, SegmentText, SegmentCircle, SegmentPoly
+from ..segments import Segment, SegmentArc, SegmentText, SegmentCircle, SegmentPoly
 
 
 resheight = 0.25      # Resistor height
@@ -39,18 +39,16 @@ class ResistorVarIEEE(ResistorIEEE):
     def __init__(self, *d, **kwargs):
         super().__init__(*d, **kwargs)
         super().__init__(**kwargs)
-        self.segments.append(SegmentArrow((1.5*reswidth, -resheight*2),
-                                          (4.5*reswidth, reswidth*3.5),
-                                          headwidth=.16, headlength=.2))
+        self.segments.append(Segment([(1.5*reswidth, -resheight*2), (4.5*reswidth, reswidth*3.5)],
+                                     arrow='->', arrowwidth=.16, arrowlength=.2))
 
 
 class ResistorVarIEC(ResistorIEC):
     ''' Variable resistor (European style) '''
     def __init__(self, *d, **kwargs):
         super().__init__(*d, **kwargs)
-        self.segments.append(SegmentArrow((1*reswidth, -resheight*2),
-                                          (5*reswidth, reswidth*3.5),
-                                          headwidth=.16, headlength=.2))
+        self.segments.append(Segment([(1*reswidth, -resheight*2), (5*reswidth, reswidth*3.5)],
+                                     arrow='->', arrowwidth=.16, arrowlength=.2))
 
 
 class Thermistor(ResistorIEC):
@@ -64,20 +62,20 @@ class PhotoresistorIEEE(ResistorIEEE):
     ''' Photo-resistor (U.S. style) '''
     def __init__(self, *d, **kwargs):
         super().__init__(*d, **kwargs)
-        self.segments.append(SegmentArrow((.7, .75), (.4, .4),
-                                          headwidth=.16, headlength=.2))
-        self.segments.append(SegmentArrow((1, .75), (.7, .4),
-                                          headwidth=.16, headlength=.2))
+        self.segments.append(Segment([(.7, .75), (.4, .4)], arrow='->',
+                                     arrowwidth=.16, arrowlength=.2))
+        self.segments.append(Segment([(1, .75), (.7, .4)], arrow='->',
+                                     arrowwidth=.16, arrowlength=.2))
 
 
 class PhotoresistorIEC(ResistorIEC):
     ''' Photo-resistor (European style) '''
     def __init__(self, *d, **kwargs):
         super().__init__(*d, **kwargs)
-        self.segments.append(SegmentArrow((.7, .75), (.4, .4),
-                                          headwidth=.16, headlength=.2))
-        self.segments.append(SegmentArrow((1, .75), (.7, .4),
-                                          headwidth=.16, headlength=.2))
+        self.segments.append(Segment([(.7, .75), (.4, .4)], arrow='->', 
+                                     arrowwidth=.16, arrowlength=.2))
+        self.segments.append(Segment([(1, .75), (.7, .4)], arrow='->',
+                                     arrowwidth=.16, arrowlength=.2))
 
 
 class Capacitor(Element2Term):
@@ -118,9 +116,8 @@ class CapacitorVar(Capacitor):
     ''' Variable capacitor '''
     def __init__(self, *d, **kwargs):
         super().__init__(*d, **kwargs)
-        self.segments.append(SegmentArrow((-2*reswidth, -resheight),
-                                          (3*reswidth, reswidth*2),
-                                          headwidth=.2, headlength=.2))
+        self.segments.append(Segment([(-2*reswidth, -resheight), (3*reswidth, reswidth*2)],
+                                     arrow='->', arrowwidth=.2, arrowlength=.2))
 
 
 class CapacitorTrim(Capacitor):
@@ -230,12 +227,10 @@ class LED(Diode):
     ''' Light emitting diode '''
     def __init__(self, *d, **kwargs):
         super().__init__(*d, **kwargs)
-        self.segments.append(SegmentArrow((resheight, resheight*1.5),
-                                          (resheight*2, resheight*3.25),
-                                          headwidth=.16, headlength=.2))
-        self.segments.append(SegmentArrow((resheight*.1, resheight*1.5),
-                                          (resheight*1.1, resheight*3.25),
-                                          headwidth=.16, headlength=.2))
+        self.segments.append(Segment([(resheight, resheight*1.5), (resheight*2, resheight*3.25)],
+                                     arrow='->', arrowwidth=.16, arrowlength=.2))
+        self.segments.append(Segment([(resheight*.1, resheight*1.5), (resheight*1.1, resheight*3.25)],
+                                     arrow='->', arrowwidth=.16, arrowlength=.2))
         self.params['lblloc'] = 'bot'
 
 
@@ -256,8 +251,8 @@ class LED2(Diode):
         
         self.segments.append(Segment(p))
         self.segments.append(Segment(p2))
-        self.segments.append(SegmentArrow(p[1], pa, headwidth=.15, headlength=.2))
-        self.segments.append(SegmentArrow(p2[1], pa2, headwidth=.15, headlength=.2))
+        self.segments.append(Segment((p[1], pa), arrow='->', arrowwidth=.15, arrowlength=.2))
+        self.segments.append(Segment((p2[1], pa2), arrow='->', arrowwidth=.15, arrowlength=.2))
         self.params['lblloc'] = 'bot'
 
 
@@ -276,8 +271,8 @@ class Photodiode(Diode):
         pa2 = Point((x[-1]-.2, y[-1]-.1)).rotate(theta)
         self.segments.append(Segment(p))
         self.segments.append(Segment(p2))
-        self.segments.append(SegmentArrow(p[-2], pa, headwidth=.15, headlength=.2))
-        self.segments.append(SegmentArrow(p2[-2], pa2, headwidth=.15, headlength=.2))
+        self.segments.append(Segment((p[-2], pa), arrow='->', arrowwidth=.15, arrowlength=.2))
+        self.segments.append(Segment((p2[-2], pa2), arrow='->', arrowwidth=.15, arrowlength=.2))
         self.params['lblloc'] = 'bot'
 
 
@@ -293,8 +288,8 @@ class PotentiometerIEEE(ResistorIEEE):
         potheight = .72
         self.anchors['tap'] = (reswidth*3, potheight)
         self.params['lblloc'] = 'bot'
-        self.segments.append(SegmentArrow((reswidth*3, potheight), (reswidth*3, reswidth*1.5),
-                                          headwidth=.15, headlength=.25))
+        self.segments.append(Segment([(reswidth*3, potheight), (reswidth*3, reswidth*1.5)],
+                                     arrow='->', arrowwidth=.15, arrowlength=.25))
 
 
 class PotentiometerIEC(ResistorIEC):
@@ -308,8 +303,8 @@ class PotentiometerIEC(ResistorIEC):
         potheight = .72
         self.anchors['tap'] = (reswidth*3, potheight)
         self.params['lblloc'] = 'bot'
-        self.segments.append(SegmentArrow((reswidth*3, potheight), (reswidth*3, reswidth*2),
-                                          headwidth=.15, headlength=.22))
+        self.segments.append(Segment([(reswidth*3, potheight), (reswidth*3, reswidth*2)],
+                                     arrow='->', arrowwidth=.15, arrowlength=.22))
 
 
 class Diac(Element2Term):
