@@ -1,4 +1,7 @@
 from .logic import And, Nand, Or, Nor, Xor, Xnor, Buf, Not, NotNot, Tgate, Schmitt, SchmittNot, SchmittAnd, SchmittNand
+from .kmap import Kmap
+from .table import Table
+from .timing import TimingDiagram
 from ..elements import Arrow, Arrowhead, Dot, Line
 
 
