@@ -6,6 +6,7 @@
 
 from __future__ import annotations
 from typing import Sequence, Any, Union
+import warnings
 import math
 
 from .types import BBox, XY, Linestyle, Capstyle, Joinstyle, Align, Arcdirection, EndRef, RotationMode
@@ -88,6 +89,10 @@ class Segment:
             capstyle: Capstyle for the segment: 'butt', 'round', 'square', ('projecting')
             joinstyle: Joinstyle for the segment: 'round', 'miter', or 'bevel'
             fill: Color to fill if path is closed
+            arrow: Arrowhead specifier, such as '->', '<-', or '<->'
+            arrowwidth: Width of arrowhead
+            arrowlength: Length of arrowhead
+            clip: Bounding box to clip to
             zorder: Z-order for segment
     '''
     def __init__(self, path: Sequence[XY],
@@ -97,13 +102,21 @@ class Segment:
                  capstyle: Capstyle=None,
                  joinstyle: Joinstyle=None,
                  fill: str=None,
+                 arrow: str=None,
+                 arrowwidth: float=0.15,
+                 arrowlength: float=0.25,
+                 clip: BBox=None,
                  zorder: int=None):
-        self.path: Sequence[XY] = path  # Untranformed path
+        self.path: Sequence[XY] = [Point(p) for p in path]   # Untranformed path
         self.zorder = zorder
         self.color = color
         self.fill = fill
         self.lw = lw
         self.ls = ls
+        self.arrow = arrow
+        self.arrowwidth = arrowwidth
+        self.arrowlength = arrowlength
+        self.clip = clip
         self.capstyle = capstyle
         self.joinstyle = joinstyle
 
@@ -120,6 +133,9 @@ class Segment:
                                   'fill': self.fill,
                                   'lw': self.lw,
                                   'ls': self.ls,
+                                  'arrow': self.arrow,
+                                  'arrowwidth': self.arrowwidth,
+                                  'arrowlength': self.arrowlength,
                                   'capstyle': self.capstyle,
                                   'joinstyle': self.joinstyle}
         style = {k: v for k, v in style.items() if k in params.keys()}
@@ -132,13 +148,16 @@ class Segment:
             Returns:
                 Bounding box limits: (xmin, ymin, xmax, ymax)
         '''
+        hw = self.arrowwidth if self.arrow else 0
         x = [p[0] for p in self.path]
         y = [p[1] for p in self.path]
-        return BBox(min(x), min(y), max(x), max(y))
+        return BBox(min(x), min(y)-hw, max(x), max(y)+hw)
 
     def doreverse(self, centerx: float) -> None:
         ''' Reverse the path (flip horizontal about the center of the path) '''
         self.path = [util.mirrorx(p, centerx) for p in self.path[::-1]]
+        if self.arrow:
+            self.arrow = self.arrow.translate(self.arrow.maketrans('<>', '><'))
 
     def doflip(self) -> None:
         ''' Vertically flip the element '''
@@ -153,7 +172,7 @@ class Segment:
                 style: Default style parameters
         '''
         path = transform.transform_array(self.path)
-
+        linepath = list(path)
         zorder = self.zorder if self.zorder is not None else style.get('zorder', 2)
         color = self.color if self.color else style.get('color', 'black')
         fill = self.fill if self.fill is not None else style.get('fill', None)
@@ -170,11 +189,32 @@ class Segment:
             elif fill is True:
                 fill = color
 
-        x = [p[0] for p in path]
-        y = [p[1] for p in path]
+        # Shrink line a bit so it doesn't extrude through arrowhead
+        if self.arrow and '<' in self.arrow:
+            delta = path[1] - path[0]
+            th = math.atan2(delta.y, delta.x)
+            linepath[0] = Point((linepath[0].x + self.arrowlength * math.cos(th),
+                                 linepath[0].y + self.arrowlength * math.sin(th)))
+        if self.arrow and '>' in self.arrow:
+            delta = path[-1] - path[-2]
+            th = math.atan2(delta.y, delta.x)
+            linepath[-1] = Point((linepath[-1].x - self.arrowlength * math.cos(th),
+                                  linepath[-1].y - self.arrowlength * math.sin(th)))
+
+        x = [p[0] for p in linepath]
+        y = [p[1] for p in linepath]
         fig.plot(x, y, color=color, fill=fill,
                  ls=ls, lw=lw, capstyle=capstyle, joinstyle=joinstyle,
-                 zorder=zorder)
+                 clip=self.clip, zorder=zorder)
+
+        if self.arrow and '<' in self.arrow:
+            theta = math.degrees(math.atan2(path[0].y-path[1].y, path[0].x-path[1].x))
+            fig.arrow(path[0], theta, color=color, zorder=zorder, clip=self.clip,
+                      arrowlength=self.arrowlength, arrowwidth=self.arrowwidth, lw=1)
+        if self.arrow and '>' in self.arrow:
+            theta = math.degrees(math.atan2(path[-1].y-path[-2].y, path[-1].x-path[-2].x))
+            fig.arrow(path[-1], theta, color=color, zorder=zorder, clip=self.clip,
+                      arrowlength=self.arrowlength, arrowwidth=self.arrowwidth, lw=1)
 
 
 class SegmentText:
@@ -191,6 +231,7 @@ class SegmentText:
             color: Color for this segment
             fontsize: Font size
             font: Font name/family
+            clip: Bounding box to clip to
             zorder: Z-order for segment
     '''
     def __init__(self, pos: Sequence[float], label: str,
@@ -200,6 +241,7 @@ class SegmentText:
                  color: str=None,
                  fontsize: float=14,
                  font: str=None,
+                 clip: BBox=None,
                  zorder: int=None):
         self.xy = pos
         self.text = label
@@ -209,6 +251,7 @@ class SegmentText:
         self.color = color
         self.rotation = rotation
         self.rotation_mode = rotation_mode
+        self.clip = clip
         self.zorder = zorder
 
     def doreverse(self, centerx: float) -> None:
@@ -233,6 +276,7 @@ class SegmentText:
                                   'color': self.color,
                                   'rotation': self.rotation,
                                   'rotation_mode': self.rotation_mode,
+                                  'clip': self.clip,
                                   'zorder': self.zorder}
 
         style = {k: v for k, v in style.items() if k in params.keys()}
@@ -264,7 +308,7 @@ class SegmentText:
             elif self.align[1] == 'top':
                 y -= h
 
-        return BBox(x-.1, y-.1, x+w+.2, y+h+.2)
+        return BBox(x-.2, y-.2, x+w+.4, y+h+.4)
 
     def draw(self, fig, transform, **style) -> None:
         ''' Draw the segment
@@ -286,7 +330,7 @@ class SegmentText:
         fig.text(self.text, xy[0], xy[1],
                  color=color, fontsize=fontsize, fontfamily=font,
                  rotation=rotation, rotation_mode=rotmode,
-                 halign=align[0], valign=align[1], zorder=zorder)
+                 halign=align[0], valign=align[1], clip=self.clip, zorder=zorder)
 
 
 class SegmentPoly:
@@ -297,9 +341,13 @@ class SegmentPoly:
             closed: Draw a closed polygon (default True)
             cornerradius: Round the corners to this radius (0 for no rounding)
             color: Color for this segment
+            fill: Color to fill if path is closed
             lw: Line width for the segment
             ls: Line style for the segment
-            fill: Color to fill if path is closed
+            hatch: Show hatch lines
+            capstyle: Capstyle for the segment: 'butt', 'round', 'square', ('projecting')
+            joinstyle: Joinstyle for the segment: 'round', 'miter', or 'bevel'
+            clip: Bounding box to clip to
             zorder: Z-order for segment
     '''
     def __init__(self, verts: Sequence[Sequence[float]],
@@ -309,19 +357,23 @@ class SegmentPoly:
                  fill: str=None,
                  lw: float=None,
                  ls: Linestyle=None,
+                 hatch: bool=False,
                  joinstyle: Joinstyle=None,
                  capstyle: Capstyle=None,
+                 clip: BBox=None,
                  zorder: int=None):
         self.verts = verts
         self.closed = closed
         self.cornerradius = cornerradius
         self.color = color
         self.fill = fill
+        self.hatch = hatch
         self.joinstyle = joinstyle
         self.capstyle = capstyle
         self.zorder = zorder
         self.lw = lw
         self.ls = ls
+        self.clip = clip
 
     def doreverse(self, centerx: float) -> None:
         ''' Reverse the path (flip horizontal about the centerx point) '''
@@ -346,6 +398,7 @@ class SegmentPoly:
                                   'lw': self.lw,
                                   'ls': self.ls,
                                   'cornerradius': self.cornerradius,
+                                  'clip': self.clip,
                                   'zorder': self.zorder}
         style = {k: v for k, v in style.items() if k in params.keys()}
         params.update(style)
@@ -391,7 +444,7 @@ class SegmentPoly:
             verts = roundcorners(verts, self.cornerradius)
 
         fig.poly(verts, closed=self.closed, color=color, fill=fill, lw=lw, ls=ls,
-                 capstyle=capstyle, joinstyle=joinstyle, zorder=zorder)
+                 hatch=self.hatch, capstyle=capstyle, joinstyle=joinstyle, clip=self.clip, zorder=zorder)
 
 
 class SegmentCircle:
@@ -404,6 +457,7 @@ class SegmentCircle:
             lw: Line width for the segment
             ls: Line style for the segment
             fill: Color to fill if path is closed. True -> fill with element color.
+            clip: Bounding box to clip to
             zorder: Z-order for segment
             ref: Flip reference ['start', 'end', None].
     '''
@@ -412,6 +466,7 @@ class SegmentCircle:
                  lw: float=None,
                  ls: Linestyle=None,
                  fill: bool | str | None=None,
+                 clip: BBox=None,
                  zorder: int=None,
                  ref: EndRef=None):
         self.center = center
@@ -421,6 +476,7 @@ class SegmentCircle:
         self.fill = fill
         self.lw = lw
         self.ls = ls
+        self.clip = clip
 
         # Reference for adding things AFTER lead extensions
         self.endref = ref
@@ -447,6 +503,7 @@ class SegmentCircle:
                                   'fill': self.fill,
                                   'lw': self.lw,
                                   'ls': self.ls,
+                                  'clip': self.clip,
                                   'ref': self.endref}
         style = {k: v for k, v in style.items() if k in params.keys()}
         params.update(style)
@@ -488,50 +545,61 @@ class SegmentCircle:
                 fill = None
             elif fill == 'bg':
                 fill = style.get('bgcolor', 'white')
-        
+
         fill = color if fill is True else None if fill is False else fill
         fig.circle(center, radius, color=color, fill=fill,
-                   lw=lw, ls=ls, zorder=zorder)
+                   lw=lw, ls=ls, clip=self.clip, zorder=zorder)
 
 
-class SegmentArrow:
-    ''' An arrow drawing segment
+class SegmentBezier:
+    ''' Quadratic or Cubic Bezier curve segment
 
         Args:
-            tail: Start coordinate of arrow
-            head: End coordinate of arrow
-            headwidth: Width of arrowhead
-            headlength: Length of arrowhead
+            p: control points (3 or 4)
             color: Color for this segment
             lw: Line width for the segment
-            ls: Line style for the segment
+            ls: Line style for the segment '-', '--', ':', etc.
+            capstyle: Capstyle for the segment: 'butt', 'round', 'square', ('projecting')
+            joinstyle: Joinstyle for the segment: 'round', 'miter', or 'bevel'
+            fill: Color to fill if path is closed
+            arrow: Arrowhead specifier, such as '->', '<-', or '<->'
+            arrowwidth: Width of arrowhead
+            arrowlength: Length of arrowhead
+            clip: Bounding box to clip to
             zorder: Z-order for segment
     '''
-    def __init__(self, tail: Sequence[float], head: Sequence[float],
-                 headwidth: float=None, headlength: float=None,
-                 color: str=None, lw: float=None,
-                 ref: EndRef=None,
+    def __init__(self, p: Sequence[XY],
+                 color: str=None,
+                 lw: float=None,
+                 ls: Linestyle=None,
+                 capstyle: Capstyle=None,
+                 arrow: str=None,
+                 arrowlength: float=.25,
+                 arrowwidth: float=.2,
+                 clip: BBox=None,
                  zorder: int=None):
-        self.tail = tail
-        self.head = head
-        self.zorder = zorder
-        self.headwidth = headwidth
-        self.headlength = headlength
+        self.p = [Point(pi) for pi in p]
+        self.arrow = arrow
         self.color = color
         self.lw = lw
-        self.endref = ref
+        self.ls = ls
+        self.capstyle = capstyle
+        self.arrowlength = arrowlength
+        self.arrowwidth = arrowwidth
+        self.clip = clip
+        self.zorder = zorder
 
     def doreverse(self, centerx: float) -> None:
         ''' Reverse the path (flip horizontal about the centerx point) '''
-        self.tail = util.mirrorx(self.tail, centerx)
-        self.head = util.mirrorx(self.head, centerx)
-        self.endref = {'start': 'end', 'end': 'start'}.get(self.endref)  # type: ignore
+        self.p = [Point(util.mirrorx(p, centerx)) for p in self.p]
+        if self.arrow:
+            self.arrow = self.arrow.translate(self.arrow.maketrans('<>', '><'))
 
     def doflip(self) -> None:
-        self.tail = util.flip(self.tail)
-        self.head = util.flip(self.head)
+        ''' Vertically flip the element '''
+        self.p = [Point(util.flip(p)) for p in self.p]
 
-    def xform(self, transform, **style) -> 'SegmentArrow':
+    def xform(self, transform, **style) -> 'SegmentBezier':
         ''' Return a new Segment that has been transformed
             to its global position
 
@@ -539,19 +607,18 @@ class SegmentArrow:
                 transform: Transformation to apply
                 style: Style parameters from Element to apply as default
         '''
-        # See https://github.com/python/mypy/issues/5382
-        # for this weird type annotation
-        params: dict[str, Any] = {'zorder': self.zorder,
-                                  'color': self.color,
+        params: dict[str, Any] = {'color': self.color,
                                   'lw': self.lw,
-                                  'headwidth': self.headwidth,
-                                  'headlength': self.headlength,
-                                  'ref': self.endref}
+                                  'ls': self.ls,
+                                  'capstyle': self.capstyle,
+                                  'arrow': self.arrow,
+                                  'arrowlength': self.arrowlength,
+                                  'arrowwidth': self.arrowwidth,
+                                  'clip': self.clip,
+                                  'zorder': self.zorder}
         style = {k: v for k, v in style.items() if k in params.keys()}
         params.update(style)
-        return SegmentArrow(transform.transform(self.tail),
-                            transform.transform(self.head),
-                            **params)
+        return SegmentBezier(transform.transform_array(self.p), **params)
 
     def get_bbox(self) -> BBox:
         ''' Get bounding box (untransformed)
@@ -559,12 +626,10 @@ class SegmentArrow:
             Returns:
                 Bounding box limits (xmin, ymin, xmax, ymax)
         '''
-        hw = self.headwidth if self.headwidth else .1
-        xmin = min(self.tail[0], self.head[0])
-        ymin = min(self.tail[1], self.head[1])
-        xmax = max(self.tail[0], self.head[0])
-        ymax = max(self.tail[1], self.head[1])
-        return BBox(xmin, ymin-hw, xmax, ymax+hw)
+        x = [p.x for p in self.p]
+        y = [p.y for p in self.p]
+        # This Might be too big, but will enclose the curve..
+        return BBox(min(x), min(y), max(x), max(y))
 
     def draw(self, fig, transform, **style) -> None:
         ''' Draw the segment
@@ -574,21 +639,19 @@ class SegmentArrow:
                 transform: Transform to apply before drawing
                 style: Default style parameters
         '''
-        tail = transform.transform(self.tail)
-        head = transform.transform(self.head)
+        p = transform.transform_array(self.p)
         zorder = self.zorder if self.zorder is not None else style.get('zorder', 1)
         color = self.color if self.color else style.get('color', 'black')
+        ls = self.ls if self.ls else style.get('ls', '-')
         lw = self.lw if self.lw else style.get('lw', 2)
-        headwidth = self.headwidth if self.headwidth else style.get('headwidth', .2)
-        headlength = self.headlength if self.headlength else style.get('headlength', .2)
-
-        fig.arrow(tail[0], tail[1], head[0]-tail[0], head[1]-tail[1],
-                  headwidth=headwidth, headlength=headlength,
-                  color=color, lw=lw, zorder=zorder)
+        capstyle = self.capstyle if self.capstyle else style.get('capstyle', 'round')
+        fig.bezier(p, color=color, lw=lw, ls=ls, capstyle=capstyle, clip=self.clip,
+                   zorder=zorder, arrow=self.arrow,
+                   arrowlength=self.arrowlength, arrowwidth=self.arrowwidth)
 
 
 class SegmentArc:
-    ''' An arc drawing segment
+    ''' An elliptical arc drawing segment
 
         Args:
             center: Center of the arc ellipse
@@ -596,12 +659,12 @@ class SegmentArc:
             height: Height of the arc ellipse
             theta1: Starting angle in degrees
             theta2: Ending angle in degrees
-            angle: Rotation of the ellipse defining the arc
             arrow: Direction of arrowhead ('cw' or 'ccw')
+            angle: Rotation of the ellipse defining the arc
             color: Color for this segment
             lw: Line width for the segment
             ls: Line style for the segment
-            fill: Color to fill if path is closed
+            clip: Bounding box to clip to
             zorder: Z-order for segment
     '''
     def __init__(self, center: Sequence[float],
@@ -612,6 +675,7 @@ class SegmentArc:
                  color: str=None,
                  lw: float=None,
                  ls: Linestyle=None,
+                 clip: BBox=None,
                  zorder: int=None):
         self.center = center
         self.width = width
@@ -623,6 +687,7 @@ class SegmentArc:
         self.color = color
         self.lw = lw
         self.ls = ls
+        self.clip = clip
         self.zorder = zorder
 
     def doreverse(self, centerx: float) -> None:
@@ -649,6 +714,7 @@ class SegmentArc:
         params: dict[str, Any] = {'color': self.color,
                                   'lw': self.lw,
                                   'ls': self.ls,
+                                  'clip': self.clip,
                                   'zorder': self.zorder}
         style = {k: v for k, v in style.items() if k in params.keys()}
         params.update(style)
@@ -702,8 +768,24 @@ class SegmentArc:
         height = self.height * transform.zoom
         fig.arc(center, width=width, height=height,
                 theta1=self.theta1, theta2=self.theta2, angle=angle,
-                color=color, lw=lw, ls=ls, zorder=zorder, arrow=self.arrow)
+                color=color, lw=lw, ls=ls, clip=self.clip, zorder=zorder, arrow=self.arrow)
 
 
-SegmentType = Union[Segment, SegmentText, SegmentPoly, SegmentArc,
-                    SegmentCircle, SegmentArrow]
\ No newline at end of file
+class SegmentArrow(Segment):
+    ''' Arrow Segment
+    
+        [DEPRECATED - use Segment with arrow parameter instead]
+    '''
+    def __init__(self, tail: Sequence[float], head: Sequence[float],
+                 headwidth: float=None, headlength: float=None,
+                 color: str=None, lw: float=None, clip: BBox=None,
+                 ref: EndRef=None, zorder: int=None):        
+        warnings.warn('SegmentArrow is deprecated. Use Segment with arrow parameter.', DeprecationWarning)
+        headwidth = 0.15 if headwidth is None else headwidth
+        headlength = 0.25 if headlength is None else headlength
+        super().__init__(path=(tail, head), arrow='->',
+                         arrowwidth=headwidth, arrowlength=headlength,
+                         color=color, lw=lw, clip=clip, zorder=zorder)
+
+
+SegmentType = Union[Segment, SegmentText, SegmentPoly, SegmentArc, SegmentCircle, SegmentBezier]
